@@ -6,7 +6,6 @@ testIndMVreg = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=N
   # This test provides a p-value PVALUE for the NULL hypothesis H0 which is
   # X is independent by TARGET given CS. The pvalue is calculated following
   # nested models
-  
   # This method requires the following inputs
   #   TARGET: a numeric matrix containing the values of the target (continuous) variable. 
   #   Its support can be R or any number betweeen 0 and 1, i.e. it contains proportions.
@@ -14,49 +13,34 @@ testIndMVreg = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=N
   #   XINDEX: the index of the variable whose association with the target we want to test. 
   #   CSINDEX: the indices if the variable to condition on. 
   #   DATAINFO: information on the structure of the data
-  
   # this method returns: the pvalue PVALUE, the statistic STAT and a control variable FLAG.
   # if FLAG == 1 then the test was performed succesfully
-  
   # References
   # [1] Norman R. Draper and Harry Smith. Applied Regression 
   # Analysis, Wiley, New York, USA, third edition, May 1998.
   # [2] Kanti V. Mardia, J. T. Kent and J. M. Bibby. Multivariate 
-  # Analysis, Academic Press, New York, USA, 1979. 
-  
-  #########################################################################################################
-  
-  #initialization
-  
+  # Analysis, Academic Press, New York, USA, 1979.
   #if the test cannot performed succesfully these are the returned values
   pvalue = log(1);
   stat = 0;
   flag = 0;
-
   csIndex[which(is.na(csIndex))] = 0;
-  if( hash )
-  {
+  if ( hash )  {
     csIndex2 = csIndex[which(csIndex!=0)]
-    csindex2 = sort(csIndex2)
+    csIndex2 = sort(csIndex2)
     xcs = c(xIndex,csIndex2)
     key = paste(as.character(xcs) , collapse=" ");
-    if( !is.null(stat_hash[[key]]) )
-    {
+    if ( !is.null(stat_hash[[key]]) )   {
       stat = stat_hash[[key]];
       pvalue = pvalue_hash[[key]];
       flag = 1;
-      
       results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
       return(results);
     }
   }
-  
-  #if the xIndex is contained in csIndex, x does not bring any new
-  #information with respect to cs
-  if( !is.na(match(xIndex,csIndex)) )
-  {
-    if( hash )#update hash objects
-    {
+  #if the xIndex is contained in csIndex, x does not bring any new information with respect to cs
+  if ( !is.na(match(xIndex,csIndex)) )   {
+    if ( hash ) {     #update hash objects
       stat_hash$key <- 0;#.set(stat_hash , key , 0)
       pvalue_hash$key <- log(1);#.set(pvalue_hash , key , 1)
     }
@@ -65,8 +49,7 @@ testIndMVreg = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=N
   }
   
   #check input validity
-  if(xIndex < 0 || csIndex < 0)
-  {
+  if(xIndex < 0 || csIndex < 0) {
     message(paste("error in testIndMVreg : wrong input of xIndex or csIndex"))
     results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
     return(results);
@@ -74,34 +57,25 @@ testIndMVreg = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=N
   
   xIndex = unique(xIndex);
   csIndex = unique(csIndex);
-  
   #extract the data
   x = dataset[ , xIndex];
   cs = dataset[ , csIndex];
-  
   #if x = any of the cs then pvalue = 1 and flag = 1.
   #That means that the x variable does not add more information to our model due to an exact copy of this in the cs, so it is independent from the target
-  if( length(cs) != 0 )
-  {
-    if( is.null(dim(cs)[2]) ) #cs is a vector
-    {
-      if( any(x != cs) == FALSE)  #if(!any(x == cs) == FALSE)
-      {
-        if( hash )#update hash objects
-        {
+  if ( length(cs) != 0 ) {
+    if ( is.null(dim(cs)[2]) )  {     #cs is a vector
+      if ( any(x != cs) == FALSE)  {    #if(!any(x == cs) == FALSE)
+        if ( hash )  {    #update hash objects
           stat_hash$key <- 0;#.set(stat_hash , key , 0)
           pvalue_hash$key <- log(1);#.set(pvalue_hash , key , 1)
         }
         results <- list(pvalue = log(1), stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
         return(results);
       }
-    }else{ #more than one var
-      for(col in 1:dim(cs)[2])
-      {
-        if(any(x != cs[,col]) == FALSE)  #if(!any(x == cs) == FALSE)
-        {
-          if( hash )#update hash objects
-          {
+    } else { #more than one var
+      for (col in 1:dim(cs)[2])  {
+        if (any(x != cs[,col]) == FALSE)  {   #if(!any(x == cs) == FALSE)
+          if ( hash )  {    #update hash objects
             stat_hash$key <- 0;#.set(stat_hash , key , 0)
             pvalue_hash$key <- log(1);#.set(pvalue_hash , key , 1)
           }
@@ -111,27 +85,9 @@ testIndMVreg = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=N
       }
     }
   }
-  
-  #if x or target is constant then there is no point to perform the test
-  if( vara( as.numeric(x) ) == 0 )
-  {
-    if( hash )#update hash objects
-    {
-      stat_hash$key <- 0;#.set(stat_hash , key , 0)
-      pvalue_hash$key <- log(1);#.set(pvalue_hash , key , 1)
-    }
-    results <- list(pvalue = log(1), stat = 0, flag = 1 , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
-    return(results);
-  }
-  
-  #trycatch for dealing with errors
-  res <- tryCatch(
-{
   #if the conditioning set (cs) is empty, we use a simplified formula
-  if(length(cs) == 0)
-  {
-    if(!is.null(univariateModels))
-    {
+  if ( length(cs) == 0 )  {
+    if ( !is.null(univariateModels) )  {
       pvalue = univariateModels$pvalue[[xIndex]];
       stat = univariateModels$stat[[xIndex]];
       flag = univariateModels$flag[[xIndex]];
@@ -139,60 +95,34 @@ testIndMVreg = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=N
       return(results);
     }
     #compute the relationship between x,target directly
-      fit2 = lm(target ~ x, weights = wei)
+    fit2 = lm(target ~ x, weights = wei)
   } else  fit2 = lm(target ~., data = as.data.frame(dataset[ , c(csIndex, xIndex)] ), weights = wei )  
-    mod = anova( fit2 ) 
-    pr = nrow( mod ) - 1
-    stat = mod[pr, 3]  ## aproximate F test
-    df1 = mod[pr, 4]
-    df2 = mod[pr, 5]
-    pvalue = pf(stat, df1, df2, lower.tail= FALSE, log.p = TRUE)
-    flag = 1;
-  
+    if ( any( is.na( coef(fit2) ) ) ) {
+    stat <- 0
+	pvalue <- log(1)
+	flag <- 1
+	} else {
+	  fit1 = lm(target ~., data = as.data.frame(dataset[ , csIndex] ), weights = wei )  
+      mod = anova( fit1, fit2 ) 
+      stat = mod[2, 5]     ## aproximate F test
+      df1 = abs(mod[2, 2])
+      df2 = mod[2, 1]
+      pvalue = pf(stat, df1, df2, lower.tail= FALSE, log.p = TRUE)
+      flag = 1;
+    }
   #last error check
-  if( is.na(pvalue) || is.na(stat) )
-  {
+  if ( is.na(pvalue) || is.na(stat) ) {
     pvalue = log(1);
     stat = 0;
     flag = 0;
-  }else{
+  } else {
     #update hash objects
-    if( hash )
-    {
-      stat_hash$key <- stat;#.set(stat_hash , key , stat)
-      pvalue_hash$key <- pvalue;#.set(pvalue_hash , key , pvalue)
+    if( hash )  {
+      stat_hash$key <- stat;         #.set(stat_hash , key , stat)
+      pvalue_hash$key <- pvalue;           #.set(pvalue_hash , key , pvalue)
     }
   }
-  
   #testerrorcaseintrycatch(4);
-  
   results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
   return(results);
-  
-},
-error=function(cond) {
-  message(paste("error in try catch of the testIndMVreg test"))
-  message("Here's the original error message:")
-  message(cond)
-  
-  #        
-  #        #for debug
-  #        printf("\nxIndex = \n");
-  #        print(xIndex);
-  #        printf("\ncsindex = \n");
-  #        print(csIndex);
-  
-  stop();
-  
-  #error case
-  pvalue = log(1);
-  stat = 0;
-  flag = 0;
-  
-  results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
-  return(results);
-},
-finally={}
-  )    
-  return(res);
 }
\ No newline at end of file
